/*
 * Jalview - A Sequence Alignment Editor and Viewer (Version 2.8.2)
 * Copyright (C) 2014 The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.ws.jabaws;

import static org.junit.Assert.*;

import java.awt.Component;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import jalview.api.AlignCalcManagerI;
import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.Annotation;
import jalview.gui.Jalview2XML;
import jalview.io.AnnotationFile;
import jalview.io.FormatAdapter;
import jalview.io.StockholmFileTest;
import jalview.ws.jws2.AADisorderClient;
import jalview.ws.jws2.Jws2Discoverer;
import jalview.ws.jws2.RNAalifoldClient;
import jalview.ws.jws2.SequenceAnnotationWSClient;
import jalview.ws.jws2.dm.JabaOption;
import jalview.ws.jws2.jabaws2.Jws2Instance;
import jalview.ws.params.AutoCalcSetting;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import compbio.metadata.WrongParameterException;

public class RNAStructExportImport
{
  public static String testseqs = "examples/unfolded_RF00031.aln";

  public static Jws2Discoverer disc;

  public static Jws2Instance rnaalifoldws;

  jalview.ws.jws2.RNAalifoldClient alifoldClient;

  public static jalview.gui.AlignFrame af = null;

  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {

    jalview.bin.Cache.initLogger();
    disc = JalviewJabawsTestUtils.getJabawsDiscoverer();

    for (Jws2Instance svc : disc.getServices())
    {

      if (svc.getServiceTypeURI().toLowerCase().contains("rnaalifoldws"))
      {
        rnaalifoldws = svc;
      }
    }

    System.out.println("State of rnaalifoldws: " + rnaalifoldws);

    if (rnaalifoldws == null)
      System.exit(0);

    jalview.io.FileLoader fl = new jalview.io.FileLoader(false);

    af = fl.LoadFileWaitTillLoaded(testseqs, jalview.io.FormatAdapter.FILE);

    assertNotNull("Couldn't load test data ('" + testseqs + "')", af);

  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
    if (af != null)
    {
      af.setVisible(false);
      af.dispose();
    }
  }

  @Test
  public void testRNAStructExport()
  {

    alifoldClient = new RNAalifoldClient(rnaalifoldws, af, null, null);

    af.getViewport().getCalcManager().startWorker(alifoldClient);

    do
    {
      try
      {
        Thread.sleep(50);
      } catch (InterruptedException x)
      {
      }
      ;
    } while (af.getViewport().getCalcManager().isWorking());

    AlignmentI orig_alig = af.getViewport().getAlignment();

    testAnnotationFileIO("Testing RNAalifold Annotation IO", orig_alig);

  }

  public static void testAnnotationFileIO(String testname, AlignmentI al)
  {
    try
    {
      // what format would be appropriate for RNAalifold annotations?
      String aligfileout = new FormatAdapter().formatSequences("PFAM",
              al.getSequencesArray());

      String anfileout = new AnnotationFile().printAnnotations(
              al.getAlignmentAnnotation(), al.getGroups(),
              al.getProperties());
      assertTrue(
              "Test "
                      + testname
                      + "\nAlignment annotation file was not regenerated. Null string",
              anfileout != null);
      assertTrue(
              "Test "
                      + testname
                      + "\nAlignment annotation file was not regenerated. Empty string",
              anfileout.length() > "JALVIEW_ANNOTATION".length());

      System.out.println("Output annotation file:\n" + anfileout
              + "\n<<EOF\n");

      // again what format would be appropriate?
      AlignmentI al_new = new FormatAdapter().readFile(aligfileout,
              FormatAdapter.PASTE, "PFAM");
      assertTrue(
              "Test "
                      + testname
                      + "\nregenerated annotation file did not annotate alignment.",
              new AnnotationFile().readAnnotationFile(al_new, anfileout,
                      FormatAdapter.PASTE));

      // test for consistency in io
      StockholmFileTest.testAlignmentEquivalence(al, al_new);
      return;
    } catch (Exception e)
    {
      e.printStackTrace();
    }
    fail("Test "
            + testname
            + "\nCouldn't complete Annotation file roundtrip input/output/input test.");
  }

  @Test
  public void testRnaalifoldSettingsRecovery()
  {
    List<compbio.metadata.Argument> opts = new ArrayList<compbio.metadata.Argument>();
    for (compbio.metadata.Argument rg : (List<compbio.metadata.Argument>) rnaalifoldws
            .getRunnerConfig().getArguments())
    {
      if (rg.getDescription().contains("emperature"))
      {
        try
        {
          rg.setValue("292");
        } catch (WrongParameterException q)
        {
          fail("Couldn't set the temperature parameter "
                  + q.getStackTrace());
        }
        opts.add(rg);
      }
      if (rg.getDescription().contains("max"))
      {
        opts.add(rg);
      }
    }
    alifoldClient = new RNAalifoldClient(rnaalifoldws, af, null, opts);

    af.getViewport().getCalcManager().startWorker(alifoldClient);

    do
    {
      try
      {
        Thread.sleep(50);
      } catch (InterruptedException x)
      {
      }
      ;
    } while (af.getViewport().getCalcManager().isWorking());
    AutoCalcSetting oldacs = af.getViewport().getCalcIdSettingsFor(
            alifoldClient.getCalcId());
    String oldsettings = oldacs.getWsParamFile();
    // write out parameters
    jalview.gui.AlignFrame nalf = null;
    assertTrue("Couldn't write out the Jar file",
            new Jalview2XML(false).SaveAlignment(af,
                    "testRnalifold_param.jar", "trial parameter writeout"));
    assertTrue("Couldn't read back the Jar file", (nalf = new Jalview2XML(
            false).LoadJalviewAlign("testRnalifold_param.jar")) != null);
    if (nalf != null)
    {
      AutoCalcSetting acs = af.getViewport().getCalcIdSettingsFor(
              alifoldClient.getCalcId());
      assertTrue("Calc ID settings not recovered from viewport stash",
              acs.equals(oldacs));
      assertTrue(
              "Serialised Calc ID settings not identical to those recovered from viewport stash",
              acs.getWsParamFile().equals(oldsettings));
      JMenu nmenu = new JMenu();
      new SequenceAnnotationWSClient().attachWSMenuEntry(nmenu,
              rnaalifoldws, af);
      assertTrue("Couldn't get menu entry for service",
              nmenu.getItemCount() > 0);
      for (Component itm : nmenu.getMenuComponents())
      {
        if (itm instanceof JMenuItem)
        {
          JMenuItem i = (JMenuItem) itm;
          if (i.getText().equals(
                  rnaalifoldws.getAlignAnalysisUI().getAAconToggle()))
          {
            i.doClick();
            break;
          }
        }
      }
      while (af.getViewport().isCalcInProgress())
      {
        try
        {
          Thread.sleep(200);
        } catch (Exception x)
        {
        }
        ;
      }
      AutoCalcSetting acs2 = af.getViewport().getCalcIdSettingsFor(
              alifoldClient.getCalcId());
      assertTrue(
              "Calc ID settings after recalculation has not been recovered.",
              acs2.getWsParamFile().equals(oldsettings));
    }
  }
}
